/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 347 (Exercise B.1.10)
**
*/

new;
library rpb,pgraph;

pqgwin many;

cls;

__eps = 1e-10;

let mu     = 5.00	6.00	 7.00	8.00	0.00	12.00;
let sigma  = 5.00	5.00	 8.00	8.00	9.00	18.00;

mu = mu / 100;
sigma = sigma / 100;

n = rows(mu);
rho = diagrv(0.25*ones(n,n),1.0);

cov = sigma .* sigma' .* rho;

print;
print "=====================  Question 1.a  =======================================";

x0 = ones(n,1)/n;
mu0 = x0'mu;
sigma0 = sqrt(x0'cov*x0);

Result0 = x0|mu0|sigma0|0;

let Target_Vol = 4.0 4.5 5.0 5.5 6.0;
Target_Vol = Target_Vol/100;
Bounds = ones(n,1) .* (0~1);

{x1,mu1,sigma1,gamma1,retcode1} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,Target_Vol,2);
tau1 = sumc(abs(x1 - x0));

Result1 = x1|mu1'|sigma1'|tau1';

print ftosa(100*(Result0~Result1),"%lf",3,2);

print;
print "=====================  Question 1.b  =======================================";

Target_Vol = seqa(3.8,14.2/200,201)/100;
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,Target_Vol,2);
tau_x = sumc(abs(x - x0));
te_x = sqrt(diag((x - x0)'cov*(x-x0)));

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xlabel("\216\202s\201(x | x[(0)]) (in %)");
  ylabel("\216\202t\201(x | x[(0)]) (in %)");
  graphprt("-c=1 -cf=app2-1-10-1.eps");
  xy(100*te_x,100*tau_x);

print;
print "=====================  Question 1.c  =======================================";

A = ones(1,n); B = 1;
Bounds = (0~1) .* ones(n,1);
gamma_ = seqa(-0.05,2/500,501);
nGamma = rows(gamma_);

{x1,mu1,sigma1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);

{mu_tilde,cov_tilde,A_tilde,B_tilde,C_tilde,D_tilde,Bounds_tilde} = Generate_Turnover_Matrices(x0,mu,cov,0,Bounds);

let tau_vector = 0.10 0.20 0.40;
nTau = rows(tau_vector);

mu2 = zeros(nGamma,nTau);
sigma2 = zeros(nGamma,nTau);
tau2 = zeros(nGamma,nTau);

i = 1;
do until i > nTau;
  D_tilde = -tau_vector[i];
  {x_tilde,mu2[.,i],sigma2[.,i],gamma2,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,C_tilde,D_tilde,Bounds_tilde,gamma_,0);
  j = 1;
  do until j > nGamma;
    x = reshape(x_tilde[.,j],3,n)'; 
    tau2[j,i] = sumc(vec(x[.,2 3]));
    j = j + 1;
  endo;
  i = i + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10|10|10|10; _pcolor = 1|2|3|4; _pltype = 6|1|3|6; _plctrl = 0|0|0|10; _pstype = 8;
  xlabel("\216\202s\201(x) (in %)");
  ylabel("\216\202m\201(x) (in %)");
  xtics(3,10,1,2);
  ytics(4,10,1,2);
  _plegstr = "Unconstrained\000\202t\201[+] = 10%\000\202t\201[+] = 20%\000\202t\201[+] = 40%";
  _plegctl = {2 5 6 2};
  _psym = 100*sigma0~100*mu0~8~3~5~1~10;
  _pmsgstr = "x[(0)]";
  _pmsgctl = 100*sigma0+0.10~100*mu0~0.18~0~1~5~8;
  graphprt("-c=1 -cf=app2-1-10-2.eps");
  xy(100*(sigma1~sigma2),100*(mu1~mu2));

print;
print "=====================  Question 1.d  =======================================";

let tau_vector = 0.10 0.20 0.40 0.80;
nTau = rows(tau_vector);

Target_Vol = 5.0/100;

x3 = zeros(n,nTau);
mu3 = zeros(nTau,1);
sigma3 = zeros(nTau,1);
tau3 = zeros(nTau,1);

i = 1;
do until i > nTau;
  D_tilde = -tau_vector[i];
  {x_tilde,mu3[i],sigma3[i],gamma3,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,C_tilde,D_tilde,Bounds_tilde,Target_Vol,2);
  x_tilde = reshape(x_tilde,3,n)'; 
  x3[.,i] = x_tilde[.,1];
  tau3[i] = sumc(vec(x_tilde[.,2 3]));
  i = i + 1;
endo;

Result3 = x3|mu3'|sigma3'|tau3';

print ftosa(100*(Result0~Result3),"%lf",3,2);

print;
print "=====================  Question 1.e  =======================================";

{x4,mu4,sigma4,gamma4,retcode4} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,Target_Vol,2);
let x4 = 0.00 0.00 0.00 0.00 1.00 0.00;
x0 = x4;
sigma0 = sqrt(x0'cov*x0);
mu0 = x0'mu;
print ftosa(100*sigma0,"sigma0 = %lf",3,2);

Target_Vol = sigma0;

tau = 0.30;
nIters = 20;

mu5 = zeros(nGamma,nIters);
sigma5 = zeros(nGamma,nIters);
mu5_path = miss(zeros(nIters+1,1),0);
sigma5_path = miss(zeros(nIters+1,1),0);

mu5_path[1] = mu0;
sigma5_path[1] = sigma0;

iter = 1;
do until iter > nIters;

  {mu_tilde,cov_tilde,A_tilde,B_tilde,C_tilde,D_tilde,Bounds_tilde} = Generate_Turnover_Matrices(x0,mu,cov,tau,0);

  Target_Vol = Target_Vol - 0.005;
  {x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,C_tilde,D_tilde,Bounds_tilde,Target_Vol,2);

  if (retcode /= 0) or (sigma_x == error(0));
    break;
  endif;

  mu5_path[iter+1] = mu_x;
  sigma5_path[iter+1] = sigma_x;

  x = reshape(x,3,n)'; 
  x0 = x[.,1];

  iter = iter + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pcolor = 1|4*ones(nIters,1); _pltype = 6|4*ones(nIters,1);
  xlabel("\216\202s\201(x) (in %)");
  ylabel("\216\202m\201(x) (in %)");
  xtics(3,10,1,2);
  ytics(0,10,1,2);
  _psym = 100*(sigma5_path~mu5_path)~ones(nIters+1,1) .* (8~2.5~4~1~10);
  _pmsgstr = "x[(0)]\000x[(1)]\000x[(2)]\000x[(3)]\000x[(4)]\000x[(5)]\000x[(6)]\000x[(7)]";
  tmp = 100*(sigma5_path+0.001~mu5_path+0.002);
  tmp = tmp[1:8,.];
  _pmsgctl = tmp~ones(8,1).*(0.20~0~1~4~8);
  graphprt("-c=1 -cf=app2-1-10-3.eps");
  xy(100*sigma1,100*mu1);

print;
print "=====================  Question 2.b  =======================================";


x0 = ones(n,1)/n;
mu0 = x0'mu;
sigma0 = sqrt(x0'cov*x0);

Target_Vol = 5/100;             
TC_p = 0.01;
TC_m = 0.02;

{x1,mu1,sigma1,gamma1,retcode1} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,bounds,Target_Vol,2);
d1 = x1 - x0;
TC1 = _max_(d1 .* TC_p,0) - _min_(d1 .* TC_m,0);

{x2,x2_m,x2_p,mu2,sigma2,gamma2,retcode2} = Compute_MVO_Portfolio_TC(x0,mu,cov,1,0,0,0,0~1,Target_Vol,2,TC_m,TC_p);

d2 = -x2_m + x2_p;
TC2 = _max_(d2 .* TC_p,0) - _min_(d2 .* TC_m,0);

err = error(0);
Results = 100*(x0~x1~x2) |
          100*(mu0~mu1~mu2) |
          100*(sigma0~sigma1~sigma2) |
          100*(err~sumc(TC1)~sumc(TC2)) |
          100*(mu0~mu1-sumc(TC1)~mu2-sumc(TC2)) ;
print ftosa(Results,"%lf",3,2);