/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 359 (Exercise B.2.8)
**
*/

new;
library rpb,pgraph;

cls;

__output = 0;

declare matrix _lambda_parameter;
declare matrix _erc_c;

/*
**> Question 1
**
*/

let sigma = 0.10 0.15 0.20 0.25;
let rho =   1.0
            0.6 1.0
            0.4 0.4 1.0
            0.3 0.3 0.2  1.0;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
n = rows(cov);

/*
**> Question 1.a
**
*/

Bounds = (0~1);

print;
print "=====================  Question 1.a  =======================================";

{x_mv,vol_mv,retcode} = Compute_MV_POrtfolio(cov,ones(1,n),1,0,0,Bounds .* ones(n,1));
{x_erc,vol_erc,mr,rc,prc} = Compute_ERC_Portfolio(0,cov);
x_ew = ones(n,1)/n;

c_mv = sumc(ln(x_mv));
c_erc = sumc(ln(x_erc));
c_ew = sumc(ln(x_ew));

print ftosa(100*(x_mv~x_erc~x_ew),"%lf",3,2);
print;
print ftosa(c_mv~c_erc~c_ew,"%lf",3,4);

print;
print "=====================  Question 1.b  =======================================";

proc _compute_mvc(x);
  local lambda,y;

  x = sqrt(x^2);
  
  lambda = _lambda_parameter;
  
  y = sqrt(x'*cov*x) - lambda*sumc(ln(x));
  retp(y);
endp;

x0 = x_ew;
_sqp_A = ones(1,n); _sqp_B = 1;
_sqp_Bounds = 0~1;

lambda = seqa(0,0.02/50,50)|seqa(0.02,0.01,151);
nIters = rows(lambda);

all_vol = zeros(nIters,1);
all_c = zeros(nIters,1);
I_star = zeros(nIters,1);
I_star_rc = zeros(nIters,1);

i = 1;
do until i > nIters;
  _lambda_parameter = lambda[i];
  {x,fmin,grd,retcode} = sqpsolve(&_compute_mvc,x0);
  all_vol[i] = sqrt(x'*cov*x);
  all_c[i] = sumc(ln(x));
  {tmp,I_star[i]} = Entropy_Index(x);
  {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
  {tmp,I_star_rc[i]} = Entropy_Index(prc);
  i = i + 1;
endo;

pqgwin many;

graphset;
  window3d;

  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  _paxht = 0.25; _pnumht = 0.20; _ptitlht = 0.275;

  setwind(1);
    _pcolor = 1;
    xlabel("\216\202l\201");
    ylabel("\216\202s\201(x[*]) (in %)");
    xtics(0,0.50,0.10,2);
    ytics(9.5,13,0.5,2);
    xy(lambda,100*all_vol);

  setwind(2);
    _pcolor = 2;
    xlabel("\216c");
    xtics(-10,-5,1,2);
    xy(all_c,100*all_vol);

  setwind(3);
    _pcolor = 4;
    xlabel("\216I[*](x[*])");
    xtics(0,4,1,2);
    xy(I_star,100*all_vol);
 
    graphprt("-c=1 -cf=app2-2-8-1.eps");

  endwind;

print;
print "=====================  Question 1.c  =======================================";
  
graphset;
  window3d;

  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  _paxht = 0.25; _pnumht = 0.20; _ptitlht = 0.275;

  setwind(1);
    _pcolor = 1;
    xlabel("\216\202l\201");
    ylabel("\216I[*]\201(RC)");
    xtics(0,0.20,0.05,2);
    ytics(1.5,4.5,0.5,2);
    _pline = 1~3~0~4~0.20~4~1~0~10;
    xy(lambda,I_star_rc);

  setwind(2);
    _pcolor = 2;
    xlabel("\216c");
    xtics(-10,-5,1,2);
    _pline = 1~3~-10~4~-5~4~1~0~10;
    xy(all_c,I_star_rc);

  setwind(3);
    _pcolor = 4;
    xlabel("\216I[*](x[*])");
    xtics(0,4,1,2);
    _pline = 1~3~0~4~4~4~1~0~10;
    xy(I_star,I_star_rc);

    graphprt("-c=1 -cf=app2-2-8-2.eps");

  endwind;

print;
print "=====================  Question 1.d  =======================================";

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6; _plwidth = 10; _pcolor = 1;

  _pcolor = 1;

  xlabel("\216\202s\201(x[*]) (in %)");
  xtics(9.5,13,0.5,2);
  ylabel("\216I[*]\201(RC)");
  ytics(1.5,4.5,0.5,2);
  _pline = 1~3~9~4~13~4~1~0~10;
  _pmsgstr = "I[*]\201(RC) = 4";
  _pmsgctl = 12.25~4.05~0.15~0~1~0~8;
  graphprt("-c=1 -cf=app2-2-8-3.eps");
  xy(100*all_vol,I_star_rc);

print;
print "=====================  Question 2.b  =======================================";

fn sigma_fn(x) = sqrt(x'cov*x);
fn Ineq_fn(x) = sumc(ln(x)) - _erc_c;

_erc_c = -10;
alpha = (_erc_c - c_erc)/n;
alpha = exp(alpha);
x0 = alpha*x_erc;

_sqp_IneqProc = &Ineq_fn;
_sqp_A = {.}; _sqp_B = {.};
_sqp_Bounds = 0~2;

{x,fmin,lagr,retcode} = sqpsolve(&sigma_fn,x0);
vol_x = sqrt(x'cov*x);
y = x / sumc(x);
vol_y = sqrt(y'cov*y);

print ftosa(100*(x_erc~x~alpha*x_erc~y),"%lf",3,2);
print;
print ftosa(100*(vol_erc~vol_x~alpha*vol_erc~vol_y),"%lf",3,4);

{vol_x,mr_x,rc_x,prc_x} = Compute_Risk_Contribution(x,cov);
Results = x~mr_x~rc_x~prc_x | (error(0)~error(0)~vol_x~error(0));

print ftosa(100*Results,"%lf",3,2);
print ftos(100*alpha,"alpha = %lf",3,2)$+"%";

print;
print "=====================  Question 2.c  =======================================";

_erc_c = 0;
alpha = (_erc_c - c_erc)/n;
alpha = exp(alpha);
x0 = alpha*x_erc;

_sqp_IneqProc = &Ineq_fn;
_sqp_A = {.}; _sqp_B = {.};
_sqp_Bounds = 0~2;

{x,fmin,lagr,retcode} = sqpsolve(&sigma_fn,x0);
vol_x = sqrt(x'cov*x);
y = x / sumc(x);
vol_y = sqrt(y'cov*y);

print ftosa(100*(x_erc~x~alpha*x_erc~y),"%lf",3,2);
print;
print ftosa(100*(vol_erc~vol_x~alpha*vol_erc~vol_y),"%lf",3,4);

{vol_x,mr_x,rc_x,prc_x} = Compute_Risk_Contribution(x,cov);
Results = x~mr_x~rc_x~prc_x | (error(0)~error(0)~vol_x~error(0));

print ftosa(100*Results,"%lf",3,2);
print ftos(100*alpha,"alpha = %lf",3,2)$+"%";





  










