/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 165 and 166 (Tables 3.2 & 3.3 and Figure 3.4)
**
*/

new;
library rpb,pgraph,optmum;

rndseed 123;

cls;

/*
**> Mean-variance optimized portfolios
**
*/

let mu =    0.05 0.09 0.07 0.06;
let sigma = 0.04 0.15 0.05 0.10;
let rho =   1.00 
            0.10  1.00
            0.40  0.20  1.00
           -0.10 -0.10 -0.20 1.00;
rho = xpnd(rho);
Cov = rho .* sigma .* sigma';

Bounds = 0~1;

gamma_x = seqa(-1,0.0025,1500);
{x,er,risk,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,1,0,0,0,Bounds,gamma_x,0); 

rfr1 = 0.03;
{x1,mu1,sigma1,gamma1,SR1} = Compute_Tangency_Portfolio(mu,cov,1,0,0,0,Bounds,rfr1,0);  

rf1 = 100*rfr1;
mu1 = 100*mu1;
sigma1 = 100*sigma1;
risk = 100*risk;
er = 100*er;

indx = minindc(Risk);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xtics(2,6,1,2);
  ytics(4,8,1,2);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  let xlab = "10" "15" "20" "25" "30";
  asclabel(xlab,0);
  _psym = risk[indx]~er[indx]~11~8~4~1~5;
  _pline = 1~3~0~er[indx]~risk[indx]~er[indx]~1~4~10 |
           1~3~risk[indx]~0~risk[indx]~er[indx]~1~4~10 ;
  _pmsgstr = "MV";
  _pmsgctl = 1.03*risk[indx]~0.995*er[indx]~0.175~0~1~4~8;  
  graphprt("-c=1 -cf=chap3-2-3-2.eps");
  xy(risk,er);

/*
**> Uniform Correlation
**
*/

let sigma = 0.04 0.06 0.08 0.10;
n = rows(sigma);

let scalar_rho = -0.20 0.00 0.20 0.50 0.70 0.90 0.99 0.9999;
nIters = rows(scalar_rho);

x1 = zeros(n,nIters);
x2 = zeros(n,nIters);
vol1 = zeros(nIters,1);
vol2 = zeros(nIters,1);

Bounds = (-5~5) .* ones(n,1);
Bounds2 = (0~1) .* ones(n,1);

iter = 1;
do until iter > nIters;
  rho = diagrv(scalar_rho[iter]*ones(n,n),1.0);
  cov = rho .* sigma .* sigma';
  call invpd(rho);
  {x1[.,iter],vol1[iter],retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,Bounds);
  {x2[.,iter],vol2[iter],retcode} = Compute_MV_Portfolio(cov,ones(1,n),1,0,0,Bounds2);
  iter = iter + 1;
endo;

x3 = ((n-1) .* scalar_rho' + 1)./(sigma^2) - (scalar_rho' ./ sigma) .* sumc(1 ./ sigma);
x3 = x3 ./ sumc(x3)';

cls;

print "-----------------------------------------------------------";
print ftosa(100*scalar_rho',"%lf",3,2);
print "-----------------------------------------------------------";
print ftosa(100*(x1|vol1'),"%lf",3,2);
print "-----------------------------------------------------------";
print ftosa(100*(x3|vol1'),"%lf",3,2);
print "-----------------------------------------------------------";
print ftosa(100*(x2|vol2'),"%lf",3,2);



