/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 217 (Computes the maximum value of the risk contribution)
**
*/

new;

// #define use_co 1;

#ifdef use_co
  library rpb,pgraph,co;
#else;
  library rpb,pgraph;
#endif

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

__output = 0;

if 1;
  FileName = "Results\\zc-cov-2003";
else;
  FileName = "Results\\zc-cov-2010";
endif;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
Yield_Rate = data[.,2:cols(data)]/100;
tau = Maturity;
tau = seqa(1,1,10);

indx = indnv(tau,Maturity);
Yield_Rate = Yield_Rate[.,indx];

Rt = Yield_Rate[rows(Yield_Rate),.]';

// PCA Factors

dt = 10;
dR = Yield_Rate - lagn(Yield_Rate,dt);
t = tau;

dR = packr(dR);
Sigma = vcx(dR);
loadm Sigma = ^FileName;
mu = meanc(dR);

let indx_Factors = 1 2 3;
nFactors = rows(indx_Factors);

{va,ve} = eighv(Sigma);
va = rev(va); 
ve = ve[.,seqa(cols(ve),-1,cols(ve))];
va = va[indx_factors]; ve = ve[.,indx_factors];
Omega = diagrv(eye(nFactors),va);
A = -ve;
D = Sigma - A*Omega*A';
P = pinv(A);

nZC = rows(t);
Nt = 100*ones(nZC,1);

Bt = exp(-t .* Rt);
Dt = t;

nB = ones(nZC,1);  // Number of ZC Bonds

Pt = Nt .* Bt;
wB =  (nB .* Pt)/sumc(nB .* Pt);
delta = - Dt .* Bt .* nB .* Nt;
wR = delta;

x0 = wR / sumc(wR);

rndseed 123;

RC_max = zeros(3,2);

iter = 1;
do until iter > 2;

  if iter == 1;
    _risk_factor_bounds = ones(nZC,1) .* (0.0~0.50);
  else;
    _risk_factor_bounds = ones(nZC,1) .* (0.0~0.25);
  endif;

  i = 1;
  do until i > 3;
    {x,y,mr,rc,prc} = Compute_RB_Factor(x0,Sigma,A,Omega,D,1.00,i,0);

    nB = 50 * x ./ (- Dt .* Bt .* Nt);
    wB =  (nB .* Pt)/sumc(nB .* Pt);
    delta = - Dt .* Bt .* nB .* Nt;
    wR = delta;

    Long_Leg = sumc(selif(nB .* Pt, nB .>= 0));
    w = (nB .* Pt)/Long_Leg;

    {vol,mr,rc,prc} = Compute_Risk_Contribution(wR,Sigma);
    RC_A = prc;

    {vol,y,mr,rc,prc} = Compute_RC_Factor(wR,Sigma,A,Omega,D);
    RC_F = prc[1:3]|sumc(prc[4:nZC]);

    RC_max[i,iter] = RC_F[i];

    i = i + 1;
  endo;

  iter = iter + 1;
endo;

print ftosa(100*RC_max,"%lf",3,2);


