/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 220 & 221 (Table 4.10 & Figure 4.12)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window_h = 2;

__output = 0;

if 1;
  FileName = "Results\\zc-cov-2003";
else;
  FileName = "Results\\zc-cov-2010";
endif;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
Yield_Rate = data[.,2:cols(data)]/100;
tau = Maturity;
tau = seqa(1,1,10);

indx = indnv(tau,Maturity);
Yield_Rate = Yield_Rate[.,indx];

Rt = Yield_Rate[rows(Yield_Rate),.]';

// PCA Factors

dt = 10;
dR = Yield_Rate - lagn(Yield_Rate,dt);
t = tau;

dR = packr(dR);
Sigma = vcx(dR);
loadm Sigma = ^FileName;
mu = meanc(dR);

let indx_Factors = 1 2 3;
nFactors = rows(indx_Factors);

{va,ve} = eighv(Sigma);
va = rev(va); 
ve = ve[.,seqa(cols(ve),-1,cols(ve))];
va = va[indx_factors]; ve = ve[.,indx_factors];
Omega = diagrv(eye(nFactors),va);
A = -ve;
D = Sigma - A*Omega*A';
P = pinv(A);

t = 2;
Ct = 100;
t1 = t; C1 = Ct;

t = 5;
Ct = 100;
t2 = t; C2 = Ct;

t = 10;
Ct = 100;
t3 = t; C3 = Ct;

let params = 0.05 -0.05 0.06 10;
fn NelsonSiegel_Bond_Price(t,Ct,params) = Bond_Price(t,Ct,NelsonSiegel_Spot_Rate(params,t));

t = t1; Ct = C1;
P = NelsonSiegel_Bond_Price(t,Ct,params);
YTM = Bond_YTM(t,Ct,P);
S = Bond_Duration(t,Ct,YTM);
P1 = P; YTM1 = YTM; S1 = S;

t = t2; Ct = C2;
P = NelsonSiegel_Bond_Price(t,Ct,params);
YTM = Bond_YTM(t,Ct,P);
S = Bond_Duration(t,Ct,YTM);
P2 = P; YTM2 = YTM; S2 = S;

t = t3; Ct = C3;
P = NelsonSiegel_Bond_Price(t,Ct,params);
YTM = Bond_YTM(t,Ct,P);
S = Bond_Duration(t,Ct,YTM);
P3 = P; YTM3 = YTM; S3 = S;

data = P1~100*YTM1~S1 |
       P2~100*YTM2~S2 |
       P3~100*YTM3~S3 ;

print ftosa(data,"%lf",3,2);

DV01 = S1|S2|S3;
Tau = 2|5|10;
Price = P1|P2|P3;

beta = 0.5;

{w1,w2,w3,w4} = Barbell_Calibrate(DV01,Price,Tau,beta);
data = w1~w2~w3~w4;
print ftosa(data,"%lf",3,3);

t = seqa(1,1,10);
Rt = NelsonSiegel_Spot_Rate(params,t);

dR = -seqa(-20,1,41)/1000;
m = rows(dR);
PnL = zeros(m,4);

V0 = sumc( data .* Price);

j = 1;
do until j > m;
  w = w1;
  PnL[j,1] = Barbell_PnL(w[1],t1,C1,w[2],t2,C2,w[3],t3,C3,t,Rt,Rt+dR[j]);
  w = w2;
  PnL[j,2] = Barbell_PnL(w[1],t1,C1,w[2],t2,C2,w[3],t3,C3,t,Rt,Rt+dR[j]);
  w = w3;
  PnL[j,3] = Barbell_PnL(w[1],t1,C1,w[2],t2,C2,w[3],t3,C3,t,Rt,Rt+dR[j]);
  w = w4;
  PnL[j,4] = Barbell_PnL(w[1],t1,C1,w[2],t2,C2,w[3],t3,C3,t,Rt,Rt+dR[j]);
  j = j + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1; 
  _pltype = 6|1|3|6; _plwidth = 10; _pcolor = 1|3|4|2; _plctrl = 0|0|0|5; _pstype = 8;
  _plegstr = "50/50\000CN\000MW\000RW";
  _plegctl = {2 6 0.5 4.5};
  xlabel("\216\202D\201r[*] (in bps)");
  ylabel("\216\202P\201");  
  xtics(-200,200,50,2);
  ytics(-0.05,0.25,0.05,2);
  graphprt("-c=1 -cf=chap4-3-1-6.eps");
  xy(1e4*dR,PnL);

proc (1) = Barbell_PnL(n1,t1,C1,n2,t2,C2,n3,t3,C3,t,Rt,new_Rt);
  local indx,R1,new_R1,R2,new_R2,R3,new_R3;
  local P1,new_P1,YTM1,D1,P2,new_P2,YTM2,D2,P3,new_P3,YTM3,D3;
  local PnL;

  indx = indnv(t1,t);
  R1 = Rt[indx];
  new_R1 = new_Rt[indx];

  indx = indnv(t2,t);
  R2 = Rt[indx];
  new_R2 = new_Rt[indx];

  indx = indnv(t3,t);
  R3 = Rt[indx];
  new_R3 = new_Rt[indx];

  P1 = Bond_Price(t1,C1,R1);
  new_P1 = Bond_Price(t1,C1,new_R1);
  YTM1 = Bond_YTM(t1,C1,P1);
  D1 = Bond_Duration(t1,C1,YTM1);

  P2 = Bond_Price(t2,C2,R2);
  new_P2 = Bond_Price(t2,C2,new_R2);
  YTM2 = Bond_YTM(t2,C2,P2);
  D2 = Bond_Duration(t2,C2,YTM2);

  P3 = Bond_Price(t3,C3,R3);
  new_P3 = Bond_Price(t3,C3,new_R3);
  YTM3 = Bond_YTM(t3,C3,P3);
  D3 = Bond_Duration(t3,C3,YTM3);

  PnL = n1*(new_P1 - P1) + n2*(new_P2-P2) + n3*(new_P3-P3);

  retp(PnL);
endp;
