/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 241 (Figure 4.23)
**
*/

new;
library rpb,pgraph;

cls;

FileName = "Results\\sovereign-bonds-egbi.fmt";
loadm egbi = ^FileName;

FileName = "Results\\sovereign-bonds-rb-egbi.fmt";
loadm egbi_rb = ^FileName;

FileName = "Results\\sovereign-bonds-debt.fmt";
loadm debt = ^FileName;

FileName = "Results\\sovereign-bonds-rb-debt.fmt";
loadm debt_rb = ^FileName;

FileName = "Results\\sovereign-bonds-gdp.fmt";
loadm gdp = ^FileName;

FileName = "Results\\sovereign-bonds-rb-gdp.fmt";
loadm gdp_rb = ^FileName;

nCountries = 11;
indx_dates = 1;
indx_x = seqa(2,1,nCountries);
indx_rc = seqa(nCountries+2,1,nCountries);
indx_vol = 2*nCountries+2;

Dates = egbi[.,indx_dates];
nDates = rows(Dates);

let string dataset = "debt" "debt_rb";

nCols = rows(dataset);

weights_France = zeros(nDates,nCols);
weights_Germany = zeros(nDates,nCols);

weights_Italy = zeros(nDates,nCols);
weights_Spain = zeros(nDates,nCols);

i = 1;
do until i > nCols;
  data = varget(dataset[i]);
  x = data[.,indx_x];
  weights_France[.,i] = x[.,4];
  weights_Germany[.,i] = x[.,5];
  weights_Italy[.,i] = x[.,8];
  weights_Spain[.,i] = x[.,11];
  i = i + 1;
endo;

avg_France = meanc(weights_France);
avg_Germany = meanc(weights_Germany);
avg_Italy = meanc(weights_Italy);
avg_Spain = meanc(weights_Spain);

weights_France = weights_France - avg_France';
weights_Germany = weights_Germany - avg_Germany';
weights_Italy = weights_Italy - avg_Italy';
weights_Spain = weights_Spain - avg_Spain';

t = seqa(1,1,ndates);

Months = trunc(Dates/100);
indx_Monthly = indexcat(Months ./= lag1(Months),1.0);
indx_Monthly = indx_Monthly|nDates;
t = t[indx_Monthly];

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pnum = 2; _pframe = 0; _pgrid = {1,1}; _pgrid = 0;
  _pltype = 6|3; _plwidth = 10; _pcolor = 1|2; _pcross = 1;
  _pnumht = 0.22; _paxht = 0.20; _ptitlht = 0.275;
  _pbartyp = {6 0, 0 4}; _pbarwid = -0.75;
  let xlab = "" "09" "" "10" "" "11" "" "12" "";
  let xlab = "" "  " "" "  " "" "  " "" "  " "";
  asclabel(xlab,0);
  xtics(1,nDates,131,0);

  setwind(1);
    title("\216France");
    ytics(-6,6,3,2);
    _pmsgstr = "09\00010\00011\00012";
    _pmsgctl = {232 -0.8 .22 0 1 0 10,
                494 -0.8 .22 0 1 0 10,
                756 -0.8 .22 0 1 0 10,
                1018 -0.8 .22 0 1 0 10};
    bar(t,100*Weights_France[indx_Monthly,.]);

  setwind(2);
    title("\216Germany");
    ytics(-10,10,5,2);
    _pmsgctl = {232 -1.4 .22 0 1 0 10,
                494 -1.4 .22 0 1 0 10,
                756 -1.4 .22 0 1 0 10,
                1018 -1.4 .22 0 1 0 10};
    bar(t,100*Weights_Germany[indx_Monthly,.]);

  setwind(3);
    title("\216Italy");
    ytics(-4,8,2,2);
    _pmsgctl = {232 -0.8 .22 0 1 0 10,
                494 -0.8 .22 0 1 0 10,
                756 -0.8 .22 0 1 0 10,
                1018 -0.8 .22 0 1 0 10};
    bar(t,100*Weights_Italy[indx_Monthly,.]);

  setwind(4);
    title("\216Spain");
    ytics(-2,4,2,2);
    _plegstr = "DEBT-WB\000DEBT-RB";
    _plegctl = {2 5 5 4.5};
    _pmsgctl = {232 -0.4 .22 0 1 0 10,
                494 -0.4 .22 0 1 0 10,
                756 -0.4 .22 0 1 0 10,
                1018 -0.4 .22 0 1 0 10};
    bar(t,100*Weights_Spain[indx_Monthly,.]);

    graphprt("-c=1 -cf=chap4-3-2-20.eps");

  endwind;

